@@ -47,6 +47,8 @@
 
 #include "sys/time.h"	/* for time_t */
 
+#include "xalloc.h"
+
 #include "mydbm.h"
 
 struct mandata {
@@ -70,12 +72,10 @@ struct mandata {
 }; 
 
 /* used by the world */
-extern inline struct mandata *dblookup_all (const char *page,
-					    const char *section,
-					    int match_case);
-extern inline struct mandata *dblookup_exact (const char *page,
-					      const char *section,
-					      int match_case);
+extern struct mandata *dblookup_all (const char *page, const char *section,
+				     int match_case);
+extern struct mandata *dblookup_exact (const char *page, const char *section,
+				       int match_case);
 extern int dbstore (struct mandata *in, const char *base);
 extern int dbdelete (const char *name, struct mandata *in);
 extern void dbprintf (const struct mandata *info);
@@ -85,10 +85,13 @@ extern void split_content (char *cont_ptr, struct mandata *pinfo);
 extern int compare_ids (char a, char b);
 
 /* local to db routines */
-extern inline void gripe_lock (char *filename);
-extern inline void gripe_corrupt_data (void);
+extern void gripe_lock (char *filename);
+extern void gripe_corrupt_data (void);
 extern datum make_multi_key (const char *page, const char *ext);
-extern inline struct mandata *infoalloc (void);
+
+/* allocate a mandata structure */
+#define infoalloc() XZALLOC (struct mandata)
+
 extern char *name_to_key (const char *name);
 extern char **split_data (char *content, char *start[]);
 extern datum make_content (struct mandata *in);
