@@ -1,7 +1,7 @@
 /*
  * manconv.c: convert manual page from one encoding to another
  *
- * Copyright (C) 2007 Colin Watson.
+ * Copyright (C) 2007, 2008 Colin Watson.
  * Based loosely on parts of glibc's iconv_prog.c, which is:
  * Copyright (C) 1998-2004, 2005, 2006, 2007 Free Software Foundation, Inc.
  *
@@ -37,128 +37,25 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #ifdef HAVE_ICONV
 #  include <iconv.h>
 #endif /* HAVE_ICONV */
 
 #include "argp.h"
-#include "dirname.h"
 
 #include "gettext.h"
 #include <locale.h>
 #define _(String) gettext (String)
-#define N_(String) gettext_noop (String)
 
 #include "manconfig.h"
 
 #include "error.h"
 #include "pipeline.h"
-#include "decompress.h"
 
-#include "encodings.h"
+#include "manconv.h"
 
-char *program_name;
-int quiet = 0;
-
-static const char *from_codes;
-static char *to_code;
-static char **from_code;
-static const char *filename;
-
-static char **split_codes (const char *codestr)
-{
-	char *codestrtok = xstrdup (codestr);
-	char *codestrtok_ptr = codestrtok;
-	char *tok;
-	size_t codearray_cur = 0, codearray_alloc = 0;
-	char **codearray = NULL;
-
-	for (tok = strsep (&codestrtok_ptr, ":"); tok;
-	     tok = strsep (&codestrtok_ptr, ":")) {
-		if (!*tok)
-			continue;	/* ignore empty fields */
-		if (codearray_cur >= codearray_alloc)
-			codearray = x2nrealloc
-				(codearray,
-				 &codearray_alloc, sizeof *codearray);
-		codearray[codearray_cur++] = xstrdup (tok);
-	}
-
-	if (codearray_cur >= codearray_alloc)
-		codearray = x2nrealloc (codearray,
-					&codearray_alloc, sizeof *codearray);
-	codearray[codearray_cur] = NULL;
-
-	free (codestrtok);
-
-	return codearray;
-}
-
-const char *argp_program_version = "manconv " PACKAGE_VERSION;
-const char *argp_program_bug_address = PACKAGE_BUGREPORT;
-error_t argp_err_exit_status = FAIL;
-
-static const char args_doc[] = N_("-f CODE[:...] -t CODE [FILENAME]");
-
-static struct argp_option options[] = {
-	{ "from-code",	'f',	N_("CODE[:...]"),
-						0,	N_("possible encodings of original text") },
-	{ "to-code",	't',	N_("CODE"),	0,	N_("encoding for output") },
-	{ "debug",	'd',	0,		0,	N_("emit debugging messages") },
-	{ "quiet",	'q',	0,		0,	N_("produce fewer warnings") },
-	{ 0, 'h', 0, OPTION_HIDDEN, 0 }, /* compatibility for --help */
-	{ 0 }
-};
-
-static error_t parse_opt (int key, char *arg, struct argp_state *state)
-{
-	switch (key) {
-		case 'f':
-			from_codes = arg;
-			return 0;
-		case 't':
-			to_code = xstrdup (arg);
-			if (!strstr (to_code, "//"))
-				to_code = appendstr (to_code, "//TRANSLIT",
-						     NULL);
-			return 0;
-		case 'd':
-			debug_level = 1;
-			return 0;
-		case 'q':
-			quiet = 1;
-			return 0;
-		case 'h':
-			argp_state_help (state, state->out_stream,
-					 ARGP_HELP_STD_HELP);
-			break;
-		case ARGP_KEY_ARG:
-			if (filename)
-				argp_usage (state);
-			filename = arg;
-			return 0;
-		case ARGP_KEY_SUCCESS:
-			if (!from_codes)
-				argp_error (state,
-					    _("must specify an input "
-					      "encoding"));
-			if (!to_code)
-				argp_error (state,
-					    _("must specify an output "
-					      "encoding"));
-			from_code = split_codes (from_codes);
-			if (!from_code || !*from_code)
-				argp_error (state,
-					    _("must specify an input "
-					      "encoding"));
-			return 0;
-	}
-	return ARGP_ERR_UNKNOWN;
-}
-
-static struct argp argp = { options, parse_opt, args_doc };
+#ifdef HAVE_ICONV
 
 static char *check_preprocessor_encoding (pipeline *p)
 {
@@ -205,9 +102,10 @@ static char *check_preprocessor_encoding (pipeline *p)
 	return pp_encoding;
 }
 
-static int try_iconv (pipeline *p, const char *try_from_code, int last)
+static int try_iconv (pipeline *p, const char *try_from_code, const char *to,
+		      int last)
 {
-	char *try_to_code = xstrdup (to_code);
+	char *try_to_code = xstrdup (to);
 	static const size_t buf_size = 65536;
 	size_t input_size = buf_size;
 	const char *input;
@@ -322,9 +220,6 @@ static int try_iconv (pipeline *p, const char *try_from_code, int last)
 			}
 		}
 
-		if (input_size < buf_size)
-			break;
-
 		input_size = buf_size;
 		input = pipeline_peek (p, &input_size);
 		while (input_size < buf_size) {
@@ -342,53 +237,39 @@ static int try_iconv (pipeline *p, const char *try_from_code, int last)
 	return ret;
 }
 
-int main (int argc, char *argv[])
+void manconv (pipeline *p, char * const *from, const char *to)
 {
-	pipeline *p;
 	char *pp_encoding;
-	char **try_from_code;
-
-	program_name = base_name (argv[0]);
-
-	if (!setlocale (LC_ALL, ""))
-		/* Obviously can't translate this. */
-		error (0, 0, "can't set the locale; make sure $LC_* and $LANG "
-			     "are correct");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	bindtextdomain (PACKAGE "-gnulib", LOCALEDIR);
-	textdomain (PACKAGE);
-
-	if (argp_parse (&argp, argc, argv, 0, 0, 0))
-		exit (FAIL);
-
-	pipeline_install_sigchld ();
-
-	if (filename) {
-		p = decompress_open (filename);
-		if (!p)
-			error (FAIL, 0, _("can't open %s"), argv[optind]);
-	} else
-		p = decompress_fdopen (dup (STDIN_FILENO));
-	pipeline_start (p);
+	char * const *try_from_code;
 
 	pp_encoding = check_preprocessor_encoding (p);
 	if (pp_encoding) {
-		try_iconv (p, pp_encoding, 1);
+		try_iconv (p, pp_encoding, to, 1);
 		free (pp_encoding);
 	} else {
-		for (try_from_code = from_code; *try_from_code;
-		     ++try_from_code)
-			if (try_iconv (p, *try_from_code,
+		for (try_from_code = from; *try_from_code; ++try_from_code)
+			if (try_iconv (p, *try_from_code, to,
 				       !*(try_from_code + 1)) == 0)
 				break;
 	}
-
-	free (to_code);
-	for (try_from_code = from_code; *try_from_code; ++try_from_code)
-		free (*try_from_code);
-	free (from_code);
-
-	pipeline_wait (p);
-
-	return 0;
 }
+
+#else /* !HAVE_ICONV */
+
+/* If we don't have iconv, there isn't much we can do; just pass everything
+ * through unchanged.
+ */
+void manconv (pipeline *p, char * const *from ATTRIBUTE_UNUSED,
+	      const char *to ATTRIBUTE_UNUSED)
+{
+	for (;;) {
+		size_t len = 4096;
+		const char *buffer = pipeline_read (p, &len);
+		if (len == 0)
+			break;
+		if (fwrite (buffer, 1, len, stdout) < len || ferror (stdout))
+			error (FATAL, 0, _("can't write to standard output"));
+	}
+}
+
+#endif /* HAVE_ICONV */
