@@ -544,10 +544,11 @@ int main (int argc, char *argv[])
 	program_name = base_name (argv[0]);
 
 	/* initialise the locale */
-	if (!setlocale (LC_ALL, ""))
+	if (!setlocale (LC_ALL, "") && !getenv ("MAN_NO_LOCALE_WARNING"))
 		/* Obviously can't translate this. */
 		error (0, 0, "can't set the locale; make sure $LC_* and $LANG "
 			     "are correct");
+	setenv ("MAN_NO_LOCALE_WARNING", "1", 1);
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	bindtextdomain (PACKAGE "-gnulib", LOCALEDIR);
 	textdomain (PACKAGE);
